// TestBuildAndLinkLibrary.cpp
//------------------------------------------------------------------------------

// Includes
//------------------------------------------------------------------------------
#include <TestFramework/UnitTest.h>

#include <Tools/FBuild/FBuildCore/FBuild.h>

#include <Core/FileIO/FileIO.h>
#include <Core/Strings/AStackString.h>

// TestBuildAndLinkLibrary
//------------------------------------------------------------------------------
class TestDLL : public UnitTest
{
private:
	DECLARE_TESTS

	void TestSingleDLL() const;
	void TestTwoDLLs() const;
	void TestTwoDLLs_NoRebuild() const;
	void TestDLLWithPCH() const;
};

// Register Tests
//------------------------------------------------------------------------------
REGISTER_TESTS_BEGIN( TestDLL )
	REGISTER_TEST( TestSingleDLL )
	REGISTER_TEST( TestTwoDLLs )
	REGISTER_TEST( TestTwoDLLs_NoRebuild )
	REGISTER_TEST( TestDLLWithPCH )
REGISTER_TESTS_END

// temporary DB names
//------------------------------------------------------------------------------
const char * g_TwoDLLS_dbFile = "../../../../ftmp/Test/DLL/twodlls.fdb";

// TestSingleDLL
//------------------------------------------------------------------------------
void TestDLL::TestSingleDLL() const
{
	FBuildOptions options;
	options.m_ConfigFile = "Data/TestDLL/fbuild.bff";
	FBuild fBuild( options );
	TEST_ASSERT( fBuild.Initialize() );

	const AStackString<> dll( "..\\..\\..\\..\\ftmp\\Test\\DLL\\dll.dll" );

	// clean up anything left over from previous runs
	FileIO::FileDelete( dll.Get() );
	TEST_ASSERT( FileIO::FileExists( dll.Get() ) == false );

	TEST_ASSERT( fBuild.Build( dll ) );

	// make sure all output files are as expecter
	TEST_ASSERT( FileIO::FileExists( dll.Get() ) );
}

// TestTwoDLLs
//------------------------------------------------------------------------------
void TestDLL::TestTwoDLLs() const
{
	FBuildOptions options;
	options.m_ConfigFile = "Data/TestDLL/fbuild.bff";
	FBuild fBuild( options );
	TEST_ASSERT( fBuild.Initialize() );

	const AStackString<> dllA( "..\\..\\..\\..\\ftmp\\Test\\DLL\\dlla.dll" );
	const AStackString<> dllB( "..\\..\\..\\..\\ftmp\\Test\\DLL\\dllb.dll" );

	// clean up anything left over from previous runs
	FileIO::FileDelete( dllA.Get() );
	FileIO::FileDelete( dllB.Get() );
	TEST_ASSERT( FileIO::FileExists( dllA.Get() ) == false );
	TEST_ASSERT( FileIO::FileExists( dllB.Get() ) == false );

	// build dllB which depends on dllA
	TEST_ASSERT( fBuild.Build( dllB ) );
	TEST_ASSERT( fBuild.SaveDependencyGraph( g_TwoDLLS_dbFile ) );

	// make sure all output files are as expecter
	TEST_ASSERT( FileIO::FileExists( dllA.Get() ) );
	TEST_ASSERT( FileIO::FileExists( dllB.Get() ) );

	// Check we build what was expected
	// 2 objs + 2 libs + 2 dlls
	const FBuildStats & stats = fBuild.GetStats();
	TEST_ASSERT( stats.GetNodesBuilt() == 6 );
}

// TestTwoDLLs_NoRebuild
//------------------------------------------------------------------------------
void TestDLL::TestTwoDLLs_NoRebuild() const
{
	FBuildOptions options;
	options.m_ConfigFile = "Data/TestDLL/fbuild.bff";
	FBuild fBuild( options );
	TEST_ASSERT( fBuild.Initialize( g_TwoDLLS_dbFile ) );

	// build again
	const AStackString<> dllB( "..\\..\\..\\..\\ftmp\\Test\\DLL\\dllb.dll" );
	TEST_ASSERT( fBuild.Build( dllB ) );

	// Check stats to be sure nothingh was built
	const FBuildStats & stats = fBuild.GetStats();
	TEST_ASSERT( stats.GetNodesBuilt() == 0 );
}

// TestDLLWithPCH
//------------------------------------------------------------------------------
void TestDLL::TestDLLWithPCH() const
{
	FBuildOptions options;
	options.m_ConfigFile = "Data/TestDLL/fbuild.bff";
	FBuild fBuild( options );
	TEST_ASSERT( fBuild.Initialize() );

	const AStackString<> dllPCH( "..\\..\\..\\..\\ftmp\\Test\\DLL\\dllpch.dll" );

	// clean up anything left over from previous runs
	FileIO::FileDelete( dllPCH.Get() );
	TEST_ASSERT( FileIO::FileExists( dllPCH.Get() ) == false );

	// build dllB which depends on dllA
	TEST_ASSERT( fBuild.Build( dllPCH ) );

	// make sure all output files are as expecter
	TEST_ASSERT( FileIO::FileExists( dllPCH.Get() ) );

	// Check we build what was expected
	// 2 objs + 1 lib + 1 dll
	const FBuildStats & stats = fBuild.GetStats();
	TEST_ASSERT( stats.GetNodesBuilt() == 4 );
}

//------------------------------------------------------------------------------
